#!/usr/bin/env python
import sqlite3
import os
import os.path


class SQL:
    def __init__(self, db_name):
        self.db_name = db_name

    def sql_create_table(self, table_name, var_dict):
        """
            table_name:database table name,str
            var_dict:variable name,dict
        """
        connect = sqlite3.connect(self.db_name)

        # Create SQL text.
        sql_text_head = 'CREATE TABLE %s \n(' % table_name
        sql_text_end = '\n);'
        sql_text = sql_text_head
        temp = ''
        for key in var_dict.keys():
            temp = key
        for k, v in var_dict.items():
            sql_text_body = '\n%s %s,' % (k, v)
            if k == temp:
                sql_text_body = sql_text_body[0:len(sql_text_body) - 1]
            sql_text = sql_text + sql_text_body
        sql_text = sql_text + sql_text_end

        # Try to create table.
        try:
            connect.execute(sql_text)
        except sqlite3.Error:
            pass
        connect.commit()
        connect.close()

    def sql_insert(self, table_name, var_key, var_values):
        """
            table_name:database table name,str
            var_key:database key name,list
            var_values:database values,list
        """
        connect = sqlite3.connect(self.db_name)

        # Change list
        var_key = tuple(var_key)
        var_key_str = str(var_key).replace("'", '')
        var_values = tuple(var_values)
        var_values_str = str(var_values)

        # Delete ',' if just one item.
        if len(var_key) == 1:
            var_key_str = var_key_str.replace(',', '')
        if len(var_values) == 1:
            var_values_str = var_values_str.replace(',', '')

        # Create SQL text
        sql_text = '''INSERT INTO %s %s VALUES %s''' % (table_name, var_key_str, var_values_str)

        # Try to insert values.
        try:
            connect.execute(sql_text)
        except sqlite3.Error:
            pass
        connect.commit()
        connect.close()

    def sql_select(self, table_name, var_key):
        """
            table_name:database table name,str
            var_key:database key name,list
            :return a list ,contains all tuples in database,list.
        """
        connect = sqlite3.connect(self.db_name)

        # Change list.
        var_key = tuple(var_key)
        var_key = str(var_key).replace("'", '')
        # Delete ()
        var_key = str(var_key)[1:len(str(var_key)) - 1]

        # Create SQL text.
        sql_text = 'SELECT %s from %s' % (var_key, table_name)

        try:
            cursor = connect.execute(sql_text)
        except sqlite3.Error:
            return 0

        result = []
        for i in cursor:
            result.append(i)
        connect.close()

        return result


class MiniSpiderSQL(SQL):
    def __init__(self, force_check=False):
        # Self.db_name = 'MiniSpider.db'.
        SQL.__init__(self, db_name=os.path.join(os.getcwd(), 'MiniSpider.db'))
        self.url_table_name = 'url_list'
        self.resource_table_name = 'resource'
        self.set_table_name = 'setup'
        # Force check.
        if force_check:
            if not os.path.isfile(os.path.join(os.getcwd(), self.db_name)):
                raise RuntimeError('Can not find database!')
        # Check database file.if not, initialize database.
        if not os.path.isfile(os.path.join(os.getcwd(), self.db_name)):
            SQL.sql_create_table(self, '%s' % self.url_table_name,
                                 var_dict={'ID': 'integer PRIMARY KEY AUTOINCREMENT', 'URL': 'TEXT UNIQUE',
                                           'STATS': 'INT'})
            SQL.sql_create_table(self, '%s' % self.resource_table_name,
                                 var_dict={'ID': 'integer PRIMARY KEY AUTOINCREMENT', 'URL': 'TEXT UNIQUE',
                                           'STATS': 'INT', 'SOURCE': 'INT'})

    def insert_url(self, url_list):
        var_key = ['URL', 'STATS']

        for index, item in enumerate(url_list):
            SQL.sql_insert(self, self.url_table_name, var_key, [item, 1])

    def insert_resource(self, resource_list, source=None):
        var_key = ['URL', 'STATS', 'SOURCE']

        for index, item in enumerate(resource_list):
            SQL.sql_insert(self, self.resource_table_name, var_key, [item, 1, source])

    def num_available_url(self):
        connect = sqlite3.connect(self.db_name)

        cursor = connect.execute("SELECT COUNT(STATS) FROM %s WHERE STATS=1" % self.url_table_name)
        result = cursor.fetchall()[0][0]

        connect.close()
        return result

    def num_available_resource(self):
        connect = sqlite3.connect(self.db_name)

        cursor = connect.execute("SELECT COUNT(STATS) FROM %s WHERE STATS=1" % self.resource_table_name)
        result = cursor.fetchall()[0][0]

        connect.close()
        return result

    def pop_url(self):
        connect = sqlite3.connect(self.db_name)

        sql_text = "SELECT ID,URL from %s WHERE STATS=1" % self.url_table_name
        cursor = connect.execute(sql_text)

        result = cursor.fetchone()

        connect.close()
        # If pop success, update this url stats.
        if result:
            self.update_url_stats(result[0], 0)
        return result

    def pop_resource(self):
        connect = sqlite3.connect(self.db_name)

        sql_text = "SELECT ID,URL,SOURCE from %s WHERE STATS=1" % self.resource_table_name
        cursor = connect.execute(sql_text)

        result = cursor.fetchone()

        connect.close()
        # If pop success, update this url stats.
        if result:
            self.update_resource_stats(result[0], 0)
        return result

    def update_url_stats(self, url_id, stats):
        connect = sqlite3.connect(self.db_name)

        sql_text = "UPDATE url_list set STATS = %s where ID = %s" % (stats, url_id)

        connect.execute(sql_text)

        connect.commit()
        connect.close()

    def update_resource_stats(self, url_id, stats):
        connect = sqlite3.connect(self.db_name)

        sql_text = "UPDATE resource set STATS = %s where ID = %s" % (stats, url_id)

        connect.execute(sql_text)

        connect.commit()
        connect.close()

    def num_all_url(self):
        connect = sqlite3.connect(self.db_name)

        cursor = connect.execute("SELECT COUNT(ID) FROM %s" % self.url_table_name)
        result = cursor.fetchall()[0][0]

        connect.close()
        return result

    def num_all_resource(self):
        connect = sqlite3.connect(self.db_name)

        cursor = connect.execute("SELECT COUNT(ID) FROM %s" % self.resource_table_name)
        result = cursor.fetchall()[0][0]

        connect.close()
        return result

    def display_all(self):
        s = 'url: %s/%s||resource: %s/%s' % (
            self.num_available_url(), self.num_all_url(), self.num_available_resource(), self.num_all_resource())
        print(s)

    def export_txt(self, file_name, table_name):
        connect = sqlite3.connect(self.db_name)

        sql_text = "SELECT URL from %s" % table_name
        cursor = connect.execute(sql_text)

        result = cursor.fetchall()

        connect.close()

        # Add '\n'.
        result = list(map(lambda x: x[0] + '\n', result))

        with open(os.path.join(os.getcwd(), file_name), mode='w') as f:
            f.writelines(result)

    def import_txt(self, file_name, table_name):
        with open(os.path.join(os.getcwd(), file_name), mode='r') as f:
            result = f.readlines()

        result = list(map(lambda x: x[:-1], result))
        if table_name == 'url_list':
            self.insert_url(result)
        elif table_name == 'resource':
            self.insert_resource(result, 'user')

    def list_url(self, table_name, num):
        connect = sqlite3.connect(self.db_name)

        sql_text = "SELECT URL from %s" % table_name
        cursor = connect.execute(sql_text)

        result = cursor.fetchall()

        connect.close()

        result = list(map(lambda x: x[0], result))

        for index, item in enumerate(result):
            if index < num:
                print(item)

    def reset(self, table_name):
        if table_name == 'u':
            for i in range(1, self.num_all_url() + 1):
                self.update_url_stats(i, 1)
        elif table_name == 'r':
            for i in range(1, self.num_all_resource() + 1):
                self.update_resource_stats(i, 1)


if __name__ == '__main__':
    MiniSpiderSQL().import_txt('1.txt', 'resource')



#!/usr/bin/env python
import os
import sys
import time
import socket
import ftplib

import urllib.parse
import urllib.request

from ssl import _create_unverified_context
from .sql import MiniSpiderSQL


class Downloader:
    """This class is used to handle the Download requests that arrive at the server."""

    def __init__(self, original_url='', filename='', work_path='', block_size=1024 * 1024, headers={}, timeout=2.0,
                 ssl_context=None,
                 terminal_mode=True, url_check=False, ftp_user='', ftp_password=''):
        # Parse chinese to ascii and delete parameters.
        if original_url == '':
            raise Exception("Downloader need URL !")
        self.url = original_url
        self.url_check = url_check
        self._check_url()
        # If filename is not specified,it is equal to the default value provided by url.
        if filename == '':
            filename = self.url.split('/')[-1]
        # Check work directory.
        self.work_path = work_path
        self._check_work_path()
        # Check headers if not exist.
        self.headers = headers
        self._check_headers()
        # Create ssl context.
        if ssl_context:
            self.ssl_context = ssl_context
        else:
            self.ssl_context = _create_unverified_context()
        # Determine whether terminal or not.
        self.terminal_mode = terminal_mode
        # Ftp username and password.
        self.ftp_user = ftp_user
        self.ftp_password = ftp_password
        # Initialization parameters.
        self.filename = filename
        self.log_filename = os.path.join(self.work_path, self.filename) + '._temp_log'
        self.block_size = int(block_size)
        self.block_list = []
        self.timeout = timeout
        self.content_length_now = 0
        self.content_length_size = 0
        # File Info.
        self.info = {}
        self.file_type = 'Unknown'
        self.server_code = 0
        # Progress speed.
        self._now_time = 0
        self._speed_pre_second = 0
        # Error detect.
        self._error_timer = 0

    def download(self):
        if self.url.split('://')[0] == 'ftp':
            self.ftp_download()
        elif self.url.split('://')[0] == 'http' or 'https':
            self.http_download()
        else:
            print('Unknown protocol!')

    def get_server_info(self):
        req = urllib.request.Request(self.url)
        if self.headers:
            for k, v in self.headers.items():
                req.add_header(k, v)

        # Add header to tell the server where to start.
        req.add_header('Range', 'bytes=0-1')

        with urllib.request.urlopen(req, context=self.ssl_context, timeout=self.timeout) as f:
            for k, v in f.getheaders():
                # Save info.
                self.info[k] = v
                # Save important info.
                if k == 'Content-Range':
                    self.content_length_size = int(v.split('/')[-1])
                if k == 'Content-Type':
                    self.file_type = v
            # Save server code.
            self.server_code = f.getcode()

    def http_download(self):
        self.get_server_info()
        # If file is completed, pass.
        if self._check_file():
            print('%s is already completed!' % self.filename)
            return True
        # If server support resume function.
        if self._support_resume() and self.info['Content-Length'] == 2 or '2':
            # Create block list.
            self._create_block_list()
            # Check log file.
            self._log_check()
            # Start download.
            for start, end in self.block_list:
                # Ignore downloaded parts.
                if start < self.content_length_now:
                    continue
                # Try download.
                self._now_time = time.time()
                self._log_save(start)
                self._download(start, end)
                # Update progress.
                self.content_length_now = end
                self._log_save(self.content_length_now)
                # Print progress.
                self._progress_bar()
            # Delete log file
            if self.content_length_now == self.content_length_size:
                self._download_success()
                self._log_delete()
        else:
            self._download(self.content_length_now, self.content_length_size)

    def ftp_download(self):
        # Extract ip address and host.
        host = self.url.split('://')[1].split('/')[0]
        ip = socket.getaddrinfo(host=host, port='21')[0][4][0]
        # Get filename and directory.
        filename = self.url.split('://')[1].split('/')[-1]
        if self.filename == '':
            self.filename = filename
        directory = self.url.split('://')[1].split('/', 1)[1].split(filename, 1)[0]
        # Begin retrieve.
        if self.ftp_user == '' and self.ftp_password == '':
            # Simple way.
            self._now_time = time.time()
            ftp = urllib.request.URLopener()
            ftp.retrieve(url, os.path.join(self.work_path, self.filename), reporthook=self._ftp_progress)
        else:
            # If ftp serve need user and password.
            with ftplib.FTP(host=ip, user=self.ftp_user, passwd=self.ftp_password) as ftp:
                with open(os.path.join(self.work_path, self.filename), mode='ab+', buffering=self.block_size) as f:
                    ftp.login()
                    ftp.cwd('%s' % directory)
                    ftp.retrbinary('RETR %s' % filename, f.write, blocksize=self.block_size)
            self._download_success()

    def _check_headers(self):
        _headers_chrome = {
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) '
                          'AppleWebKit/537.36 (KHTML, like Gecko) Chrome/56.0.2924.87 Safari/537.36'
        }
        if self.headers == {}:
            self.headers = _headers_chrome

    def _log_save(self, content_length):
        content = [content_length]
        file = open(self.log_filename, mode='w')
        for index, item in enumerate(content):
            file.write(str(content[index]) + '\n')
        file.close()

    def _log_read(self):
        result = []

        with open(self.log_filename, 'r') as content:
            for line in content:
                if line[-1] == '\n':
                    line = line[0:-1]
                result.append(line)

        return result[0]

    def _log_delete(self):
        os.remove(self.log_filename)

    def _log_check(self):
        if os.path.isfile(self.log_filename):
            self.content_length_now = int(self._log_read())
        else:
            self._log_save(0)

    def _error(self):
        self._error_timer += 1
        if self._error_timer == 5:
            self._error_timer = 0
            time.sleep(1)

    def _download_success(self):
        # Delete redundant character.
        if self.terminal_mode:
            sys.stdout.write('                                                                        \r')
            sys.stdout.flush()
        print('%s completed' % self.filename)

    def _support_resume(self):
        if self.server_code == 0:
            self.get_server_info()
        if self.server_code == 206:
            return True
        else:
            return False

    def _create_block_list(self):
        # Attention: request length is closed interval, but python array is different.
        block_number = int((self.content_length_size - self.content_length_now) / self.block_size) + 1
        l = [x * self.block_size for x in range(0, block_number)]
        if l[-1] > self.content_length_size:
            l[-1] = self.content_length_size + 1
        else:
            l.append(self.content_length_size)
        for i in range(1, len(l)):
            temp = [l[i - 1], l[i]]
            self.block_list.append(tuple(temp))

    def _progress_bar(self):
        if self.content_length_size == 0:
            return False
        progress = (self.content_length_now / self.content_length_size)
        size_mb = self.content_length_size / (1024 * 1024)
        _time = (time.time() - self._now_time)
        self._speed_pre_second = int(self.block_size) / (_time * 1024 * 1024)
        # Print in terminal or python interpreter.
        if self.terminal_mode:
            s_terminal = 'All:%.2f mb,%.2f%% , speed: %.2fmb/s \r' % (size_mb, progress * 100, self._speed_pre_second)
            sys.stdout.write(s_terminal)
            sys.stdout.flush()
        else:
            s = 'All:%.2f mb,%.2f%% , speed: %.2fmb/s' % (size_mb, progress * 100, self._speed_pre_second)
            print(s)

    def _download(self, begin, end):
        req = urllib.request.Request(self.url)
        if self.headers:
            for k, v in self.headers.items():
                req.add_header(k, v)
        byte_range = 'bytes=' + str(begin) + '-' + str(end - 1)
        req.add_header('Range', byte_range)
        with urllib.request.urlopen(req, context=self.ssl_context, timeout=self.timeout) as r:
            with open(os.path.join(self.work_path, self.filename), mode='ab+', buffering=self.block_size) as f:
                f.seek(self.content_length_now)
                f.truncate()
                f.write(r.read())

    def _ftp_progress(self, _chunk_num, _chunk_size, _file_size):
        self.block_size = _chunk_size
        self.content_length_now = _chunk_num * _chunk_size
        self.content_length_size = _file_size
        if self.content_length_now > self.content_length_size:
            self.content_length_now = self.content_length_size
            self._download_success()
        else:
            self._progress_bar()
        self._now_time = time.time()

    def _check_file(self):
        try:
            file_size = os.path.getsize(os.path.join(self.work_path, self.filename))
            if file_size == self.content_length_size:
                return True
            else:
                return False
        except FileNotFoundError:
            return False

    def _check_work_path(self):
        if self.work_path == '':
            return True
        try:
            os.chdir(self.work_path)
        except FileNotFoundError:
            os.mkdir(self.work_path)
        if self.work_path[-1] != '/':
            self.work_path = self.work_path + '/'

    def _check_url(self):
        if self.url_check:
            # Delete get parameters.
            self.url = self.url.split('?', 1)[0]
            # Add protocol if not exist.
            if self.url.split('://', 1)[0] != 'http' or 'https':
                self.url = 'http://' + self.url
        # Parse chinese to ascii.
        self.url = urllib.parse.quote(self.url, safe='/:?=@&[]')


class MiniSpiderDownloader():
    def __init__(self):
        self.SQL = MiniSpiderSQL()
        self.work_path = os.getcwd()

    def start(self, work_path=None, classify=True, timeout=2.0):
        # If work path provided, use it.
        if work_path is not None:
            self.work_path = work_path

        while self.SQL.num_available_resource():
            # Pop resource from database.
            id, url, source = self.SQL.pop_resource()

            # Download.
            try:
                # Check classify
                if classify:
                    Downloader(url, work_path=os.path.join(self.work_path, str(source)), timeout=timeout).download()
                else:
                    Downloader(url, work_path=self.work_path, timeout=timeout).download()
            except Exception as e:
                print(e)
                self.SQL.update_resource_stats(url_id=id, stats=1)


if __name__ == "__main__":
    pass



#!/usr/bin/env python
import json
import re
import os

from .sql import MiniSpiderSQL


class Extractor:
    def __init__(self, content=None):
        self.content = content
        self.SQL = MiniSpiderSQL()

    def run_extractor(self, pattern, mode, source, host=None):
        result = []
        match_list = re.findall(pattern, self.content)
        for i in match_list:
            # If it is a href,add host.
            if host:
                i = host + i
            result.append(i)
        if mode == 'resource':
            self.SQL.insert_resource(result, source)
        elif mode == 'url':
            self.SQL.insert_url(result)

    def make_extractor(self, extractor_name=None, pattern='', mode=''):
        if pattern and mode:
            host = None
            # If extract href url.
            if len(pattern) == 2:
                host = pattern[1]
                pattern = pattern[0]
            # Make json data.
            if host:
                info = {
                    'pattern': pattern,
                    'mode': mode,
                    'host': host
                }
            else:
                info = {
                    'pattern': pattern,
                    'mode': mode
                }
            s = json.dumps(info)
            # If name is None, make a name.
            if extractor_name is None:
                extractor_list = self._find_all_extractor()
                for i in range(1, 100):
                    extractor_name = str(i) + '.extractor'
                    if extractor_name not in extractor_list:
                        break
            else:
                # Add suffix name.
                extractor_name = extractor_name + '.extractor'
            # Save extractor.
            with open(extractor_name, mode='w') as f:
                f.write(s)
        else:
            return False

    def run_all_extractor(self, source=None):
        extractor_list = self._find_all_extractor()
        for name in extractor_list:
            with open(name, mode='r') as f:
                info = json.loads(f.read())
                # Check href extractor.
                if len(info) == 3:
                    self.run_extractor(info['pattern'], info['mode'], source, info['host'])
                else:
                    self.run_extractor(info['pattern'], info['mode'], source)

    @staticmethod
    def _find_all_extractor():
        file_list = os.listdir(os.getcwd())
        extractor_list = []
        for i in file_list:
            try:
                suffix_name = i.rsplit('.')[1]
            except IndexError:
                continue
            if suffix_name == 'extractor':
                extractor_list.append(i)

        return extractor_list



#!/usr/bin/env python
import re
import urllib.request
import urllib.parse
import difflib
import json

from ssl import _create_unverified_context
from .sql import MiniSpiderSQL
from .extractor import Extractor


class MiniSpider:
    def __init__(self, original_url=None, timeout=2, search=(), ssl_context=None, url_check=True,
                 similarity_threshold=0.6,
                 display_number=100):
        # Parse chinese to ascii and delete parameters.
        self.url = original_url
        if self.url:
            self.url_check = url_check
            self._check_url()
            self.host = 'http://' + self.url.split('//')[1].split('/')[0]
        # Create ssl context.
        if ssl_context:
            self.ssl_context = ssl_context
        else:
            self.ssl_context = _create_unverified_context()
        # Initialization parameters.
        self.temp_file_name = 'mini-spider.temp'
        self.timeout = timeout
        self.similarity_threshold = similarity_threshold
        self.pattern_list = []
        self.search_list = self._initialize_search(search)
        self.display_number = display_number
        self.result = []
        self.http_flag = 0

    def _url_read(self, url=None):
        if url is None:
            url = self.url

        req = urllib.request.Request(url)
        with urllib.request.urlopen(req, context=self.ssl_context, timeout=self.timeout) as r:
            return self._content_decode(r.read())

    def _check_url(self):
        if self.url_check:
            # Add protocol if not exist.
            if self.url.split('://', 1)[0] != 'http' or 'https':
                self.url = 'http://' + self.url
        # Parse chinese to ascii.
        self.url = urllib.parse.quote(self.url, safe='/:?=@&[]')

    def _handle_match(self, match_list):
        """ Handle match list by similarity threshold and add it to result."""
        # If match_list = [], return False.
        if len(match_list) == 0:
            return False

        # Eliminate duplicate and sort().
        match_list = self.duplicate_eliminate(match_list)
        match_list.sort()

        # If match_list only have one item.
        if len(match_list) == 1:
            self.result.append(match_list)
            return True

        # Handle.
        temp = []
        flag = 0

        for index, item in enumerate(match_list):
            if self.similar(match_list[flag], item) >= self.similarity_threshold:
                temp.append(item)
                continue
            else:
                flag = index
                self.result.append(temp)
                temp = []
                temp.append(item)
        if len(temp):
            self.result.append(temp)

    def _display_result(self):
        for index, item in enumerate(self.result):
            print('[%s]:' % index)
            flag = 1
            for i, j in enumerate(item):
                if i >= self.display_number:
                    if flag:
                        print('%s is not displayed' % (len(item) - self.display_number))
                        flag = 0
                    continue
                # Print href url.
                if index >= self.http_flag:
                    j = self.host + j
                print('---(%s)%s' % (i, j))

    def _pattern_make_http(self):
        for i in self.search_list:
            temp = "http://\S+?\." + i
            self.pattern_list.append(temp)

    def _pattern_make_href(self):
        for i in self.search_list:
            temp = 'href="(/\S+?\.%s)"' % i
            self.pattern_list.append(temp)

    @staticmethod
    def similar(str1, str2):
        if str1 == str2:
            return 1
        return float(difflib.SequenceMatcher(None, str1, str2).ratio())

    @staticmethod
    def _content_decode(content):
        try:
            return content.decode('utf-8')
        except UnicodeDecodeError:
            try:
                return content.decode('gbk')
            except UnicodeDecodeError:
                return content.decode('gb2312')

    @staticmethod
    def _initialize_search(search):
        temp = []
        if type(search) == str:
            temp.append(search)
            return temp
        for i in search:
            temp.append(i)
        return temp

    def find_longest_size(self, match_list):
        # If only one item, return.
        if len(match_list) == 1:
            return len(match_list[0])

        size = len(match_list[0])
        for index, item in enumerate(match_list):
            if index == len(match_list) - 1:
                break
            temp = self._find_longest_match(item, match_list[index + 1])
            if temp < size:
                size = temp

        return size

    @staticmethod
    def _find_longest_match(str1, str2):
        len_1 = len(str1)
        len_2 = len(str2)

        if len_1 >= len_2:
            length = len_2
        else:
            length = len_1

        for i in range(0, length):
            if str1[i] == str2[i]:
                continue
            else:
                return i

    @staticmethod
    def _is_letter(match_str):
        pattern = '[a-z]'
        if re.findall(pattern, match_str):
            return True
        else:
            return False

    @staticmethod
    def _is_letter_capital(match_str):
        pattern = '[A-Z]'
        if re.findall(pattern, match_str):
            return True
        else:
            return False

    @staticmethod
    def _is_number(match_str):
        pattern = '[0-9]'
        if re.findall(pattern, match_str):
            return True
        else:
            return False

    @staticmethod
    def _get_suffix_name(_url):
        return _url.rsplit('.', 1)[1]

    @staticmethod
    def duplicate_eliminate(list_input):
        """ Delete all duplicate in a list."""
        result = []
        for i in list_input:
            if i not in result:
                result.append(i)

        return result

    def _save_temp(self, content_list):
        s = json.dumps(content_list)
        with open(self.temp_file_name, mode='w') as f:
            f.write(s)

    def _read_temp(self):
        with open(self.temp_file_name, mode='r') as f:
            result = json.loads(f.read())
        return result

    def analysis_url(self):
        # Read URL content.
        content = self._url_read()

        # Make http pattern.
        self._pattern_make_http()
        # Match http pattern.
        match_list = []
        for i in self.pattern_list:
            match = re.findall(i, content)
            for j in match:
                match_list.append(j)
            # Classify the url into certain types.
            self._handle_match(match_list)
            match_list = []
        # Get length of http pattern match list. First item is href item.
        self.http_flag = len(self.result)

        # Delete http pattern.
        self.pattern_list = []

        # Make href pattern.
        self._pattern_make_href()
        # Match href pattern.
        match_list = []
        for i in self.pattern_list:
            match = re.findall(i, content)
            for j in match:
                match_list.append(j)
            # Classify the url into certain types.
            self._handle_match(match_list)
            match_list = []

        # Check if not match.
        if not self.result:
            # Do some thing.
            print('Error!We find nothing!')
            return False

        # Save result、http_flag、host in temp file.
        temp = {
            'result': self.result,
            'http_flag': self.http_flag,
            'host': self.host
        }
        self._save_temp(temp)

        # Print result.
        self._display_result()

    def make_specific_http_pattern(self, specific_block_list):
        """Make specific pattern for entire URL."""
        # Get longest match block.
        same_size = self.find_longest_size(specific_block_list)

        # Get same_block.
        same_block = specific_block_list[0][0:same_size]

        # Get suffix name and add regular expression format.
        suffix_name = '\.' + self._get_suffix_name(specific_block_list[0])

        # Split same block.
        header = same_block.split('//')[0] + '//'
        latter_part = same_block.split('//')[1]

        # If only one item,delete suffix name.
        if len(same_block) == len(specific_block_list[0]):
            latter_part = latter_part[0:len(latter_part) - len(self._get_suffix_name(specific_block_list[0])) - 1]

        # Split latter part.
        latter_part_list = latter_part.split('/')
        host = latter_part_list[0]

        # Make specific pattern.
        last_block = ''
        for index, item in enumerate(latter_part_list):
            temp = []
            if index == 0:
                continue
            for j in item:
                if self._is_letter(j):
                    temp.append('[a-z]')
                elif self._is_letter_capital(j):
                    temp.append('[A-Z]')
                elif self._is_number(j):
                    temp.append('[0-9]')
                else:
                    temp.append(j)
            # Format list.
            pass

            pattern_block = ''.join(temp)
            last_block = last_block + '/' + pattern_block
        # Check if need supplement.
        if len(same_block) == len(specific_block_list[0]):
            char_supplement = ''
        else:
            char_supplement = '\S+?'

        result_pattern = header + host + last_block + char_supplement + suffix_name

        return result_pattern

    def make_specific_href_pattern(self, specific_block_list, host):
        """Make specific pattern for href URL."""
        # Get longest match block.
        same_size = self.find_longest_size(specific_block_list)

        # Get same_block and it's length.
        same_block = specific_block_list[0][0:same_size]
        same_length = len(same_block)

        # Get suffix name and add regular expression format.
        suffix_name = '\.' + self._get_suffix_name(specific_block_list[0])

        # If only one item,delete suffix name.
        if same_length == len(specific_block_list[0]):
            same_block = same_block[0:same_length - len(self._get_suffix_name(specific_block_list[0])) - 1]

        # Make specific pattern.
        main_block = ''
        temp_block = ''
        for index, item in enumerate(same_block):
            temp = []
            if self._is_letter(item):
                temp.append('[a-z]')
            elif self._is_letter_capital(item):
                temp.append('[A-Z]')
            elif self._is_number(item):
                temp.append('[0-9]')
            else:
                temp.append(item)
            main_block = main_block + temp_block.join(temp)

        # Check if need supplement.
        if same_length == len(specific_block_list[0]):
            char_supplement = ''
        else:
            char_supplement = '\S+?'

        pattern = '''href="(''' + main_block + char_supplement + suffix_name + ''')"'''

        return pattern, host

    def choose_block(self, num, start=None, end=None):
        # Choose block to make specific pattern. +1 used fix python array problem.
        block = self._read_temp()['result'][num]
        http_flag = self._read_temp()['http_flag']
        host = self._read_temp()['host']

        # Make href pattern.
        if num >= http_flag:
            if start is not None and end is not None:
                specific_pattern = self.make_specific_href_pattern(block[start:end + 1], host)
            elif start is not None:
                specific_pattern = self.make_specific_href_pattern(block[start:start + 1], host)
            else:
                specific_pattern = self.make_specific_href_pattern(block[0:], host)
        # Make http pattern.
        else:
            if start is not None and end is not None:
                specific_pattern = self.make_specific_http_pattern(block[start:end + 1])
            elif start is not None:
                specific_pattern = self.make_specific_http_pattern(block[start:start + 1])
            else:
                specific_pattern = self.make_specific_http_pattern(block[0:])

        return specific_pattern

    def start(self, url=None):
        # 1.Get url.
        if url is None:
            # If url is not provided, use SQL data.
            try:
                flag, url = MiniSpiderSQL().pop_url()
            except TypeError:
                raise Exception('Please input original url!')

        # 2.Read content.
        try:
            content = self._url_read(url)
            Extractor(content).run_all_extractor(0)
        except Exception as e:
            print(e)
        MiniSpiderSQL().display_all()

        # 3.Loop.
        while MiniSpiderSQL().num_available_url():
            flag, url = MiniSpiderSQL().pop_url()

            try:
                content = self._url_read(url)
                Extractor(content).run_all_extractor(flag)
            except Exception as e:
                MiniSpiderSQL().update_url_stats(flag,1)
                print(e)

            MiniSpiderSQL().display_all()



#!/usr/bin/env python
import argparse

from .sql import MiniSpiderSQL
from .scheduler import MiniSpider
from .extractor import Extractor
from .downloader import MiniSpiderDownloader

__version__ = '0.0.2.1'


def main():
    # Make parser for terminal.
    description = 'MiniSpider makes it easy to create user-friendly spider.'
    usage = 'mini-spider [OPTION]... [URL]...'
    parser = argparse.ArgumentParser(prog='MiniSpider', description=description, usage=usage,
                                     epilog='Powered by ZYunH. Version:%s' % __version__)

    # Add arguments.
    analysis_help = 'Analysis a URL.'
    parser.add_argument('-a', help=analysis_help, nargs='+', dest='analysis_url', metavar='[URL]')

    similarity_threshold_help = 'Set similarity_threshold,default = 0.6'
    parser.add_argument('-st', help=similarity_threshold_help, nargs=1, dest='similarity', type=float,
                        metavar='[float]')

    choose_help = 'Choose block make extractor.'
    parser.add_argument('-c', help=choose_help, nargs='+', dest='choose_block', type=int, metavar='[num]')

    timeout_help = 'Set timeout.(default: 2)'
    parser.add_argument('-time', help=timeout_help, nargs=1, dest='time_out', type=float, metavar='[float]')

    to_help = 'Choose match data.(default: u)'
    parser.add_argument('-to', help=to_help, nargs='?', dest='to', const='u', choices=['u', 'r'])

    name_help = 'Name your extractor.it can be ignored.'
    parser.add_argument('-n', help=name_help, nargs=1, dest='name', metavar='[name]')

    start_help = 'Start spider to get url and resource.'
    parser.add_argument('-start', help=start_help, nargs='?', dest='start', const=True, metavar='URL')

    download_help = 'Download all url from database.'
    parser.add_argument('-download', help=download_help, nargs='?', dest='download', const=True, metavar='Path')

    make_help = 'Make extractor by user.'
    parser.add_argument('-m', help=make_help, nargs='+', dest='make', metavar='[RE]')

    export_help = 'Export url from database.'
    parser.add_argument('-export', help=export_help, nargs=1, dest='export_url', metavar='[FileName]')

    import_help = 'Import url into database.'
    parser.add_argument('-import', help=import_help, nargs=1, dest='import_url', metavar='[FileName]')

    list_help = 'List url in url_list or resource.options: "u" or "r"'
    parser.add_argument('-list', help=list_help, nargs='+', dest='list_url', metavar='')

    false_help = 'Disable classification function in -download.'
    parser.add_argument('-false', help=false_help, nargs='?', dest='false_set', const=True, metavar='')

    reset_help = 'Reset database stats = 1.(default: u)'
    parser.add_argument('-reset', help=reset_help, nargs='?', dest='reset', const='u', choices=['u', 'r'])

    # Parse arguments.
    args = parser.parse_args()

    # Parse analysis url.
    if args.analysis_url:
        if len(args.analysis_url) == 1:
            print('Error: Please input what resource you are looking for!')
            return False
        timeout = 2.0
        if args.time_out:
            timeout = args.time_out[0]
        if args.similarity:
            spider = MiniSpider(args.analysis_url[0], search=args.analysis_url[1:],
                                similarity_threshold=args.similarity[0], timeout=timeout)
            spider.analysis_url()
        else:
            spider = MiniSpider(args.analysis_url[0], search=args.analysis_url[1:], similarity_threshold=0.6,
                                timeout=timeout)
            spider.analysis_url()

    # Choose block make regular expression.
    elif args.choose_block:
        num = args.choose_block[0]
        start = None
        end = None

        if len(args.choose_block) == 2:
            start = args.choose_block[1]
        elif len(args.choose_block) == 3:
            start = args.choose_block[1]
            end = args.choose_block[2]

        pattern = MiniSpider().choose_block(num, start, end)

        # Print pattern.
        if len(pattern) == 2:
            print('Host:' + pattern[1])
            print(pattern[0])
        else:
            print(pattern)

        # Choose database.
        if args.to:
            name = None
            if args.name:
                name = args.name[0]
            if args.to[0] == 'u':
                Extractor().make_extractor(name, pattern=pattern, mode='url')
            elif args.to[0] == 'r':
                Extractor().make_extractor(name, pattern=pattern, mode='resource')
            print('The extractor was created successfully！')
        else:
            print("Error: Please input  '-to u' or '-to r'")
            return False

    # Make pattern by user.
    elif args.make:
        pattern_user = args.make[0]

        # Get host, if possible.
        if len(args.make) == 2:
            pattern_user = pattern_user, args.make[1]

        # Choose database.
        if args.to:
            name = None
            if args.name:
                name = args.name[0]
            if args.to[0] == 'u':
                Extractor().make_extractor(name, pattern=pattern_user, mode='url')
            elif args.to[0] == 'r':
                Extractor().make_extractor(name, pattern=pattern_user, mode='resource')
            print('The extractor was created successfully！')
        else:
            print("Error: Please input  '-to u' or '-to r'")
            return False

    # Start project.
    elif args.start:
        if args.start is True:
            MiniSpider().start()
        else:
            MiniSpider().start(args.start)

    # Start downloading.
    elif args.download:
        classify = True
        timeout = 2.0
        if args.false_set:
            classify = False
        if args.time_out:
            timeout = args.time_out[0]
        if args.download is True:
            MiniSpiderDownloader().start(classify=classify, timeout=timeout)
        else:
            MiniSpiderDownloader().start(args.download, classify=classify, timeout=timeout)

    # Import txt.
    elif args.import_url:
        # Choose database.
        if args.to:
            if args.to[0] == 'u':
                MiniSpiderSQL().import_txt(args.import_url[0], 'url_list')
            elif args.to[0] == 'r':
                MiniSpiderSQL().import_txt(args.import_url[0], 'resource')
            print('Import success!')
        else:
            print("Error: Please input  '-to u' or '-to r'")
            return False

    # Export txt.
    elif args.export_url:
        # Choose database.
        if args.to:
            if args.to[0] == 'u':
                MiniSpiderSQL().export_txt(args.export_url[0], 'url_list')
            elif args.to[0] == 'r':
                MiniSpiderSQL().export_txt(args.export_url[0], 'resource')
            print('Export success!')
        else:
            print("Error: Please input  '-to u' or '-to r'")
            return False

    # List database.
    elif args.list_url:
        num = 50
        if len(args.list_url) == 2:
            num = int(args.list_url[1])
        if args.list_url[0] == 'u':
            MiniSpiderSQL().list_url(table_name='url_list', num=num)
        elif args.list_url[0] == 'r':
            MiniSpiderSQL().list_url(table_name='resource', num=num)
        else:
            print("Error: Please input  '-list u' or '-list r'")
            return False

            # print(parser.print_help())

    # Reset database stats.
    elif args.reset:
        MiniSpiderSQL().reset(args.reset[0])

    else:
        parser.print_help()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
import os
import json

from setuptools import setup, find_packages

PROJECT_NAME = 'mini-spider'
MODULE_NAME = 'minispider'

here = os.path.abspath(os.path.dirname(__file__))
project_info = json.loads(open(os.path.join(here, 'mini-spider.json')).read())

setup(
    name=project_info['name'],
    version=project_info['version'],

    author=project_info['author'],
    author_email=project_info['author_email'],

    description=project_info['description'],
    long_description=project_info['long_description'],
    keywords=project_info['keywords'],

    url=project_info['url'],
    license=project_info['license'],

    platforms='any',

    packages=find_packages(),
    include_package_data=True,

    entry_points={
        'console_scripts': [
            'mini-spider = minispider.__main__:main'
        ]
    }

)
